--[[
Copyright (C) 2016  Neemys

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

See <LICENSE.txt> for a copy of this license, or see <http://www.gnu.org/licenses/>
]]


require("misc")
require("config")

-- Look if the building is entirely on a platform.
-- Return true if it is not on platform
local function checkBuildingNotOnPlatform(entity,box)
	for x=box.left_top.x,box.right_bottom.x,1 do
		for y=box.left_top.y,box.right_bottom.y,1 do
			local tile = entity.surface.get_tile(x,y)
			local xHidden = x
			local yHidden = y
			local hiddenTileName = entity.surface.get_hidden_tile({xHidden,yHidden})
			if(tile and tile.name ~= buildingplatform.tileName and (hiddenTileName and hiddenTileName ~= buildingplatform.tileName or not hiddenTileName)) then
				return true
			end
		end
	end
	return false
end
-- When player build something, check if it is a restricted building and build on a platform.
-- If not, remove it. 
script.on_event(defines.events.on_built_entity, function(event)
	local player = game.players[event.player_index]
	local entity = event.created_entity
	local is_ghost = (entity.name == "entity-ghost");
	local entity_prototype = (is_ghost and entity.ghost_prototype or entity.prototype );
	--check if placed building is blocked outside building platform
	local isBlocked = false
	for key, value in pairs(entity_prototype.collision_mask) do
		if key == "layer-15" and value then
			isBlocked = true
			break
		end
	end
	if(isBlocked) then
		if(checkBuildingNotOnPlatform(entity,entity.bounding_box)) then
			-- when removing, we need to loot it in case another mod put item inside
			-- need a solution if some item can't go inside inventory, maybe spawn a chest ? For the moment they just vanish
			if(not is_ghost) then
				local loot = entity_prototype.loot
				if loot then
					for i,loot_ele in pairs(loot) do
						player.insert({name=loot.item,count=loot.count_max})
					end
				end
				for i,item in pairs(entity.prototype.mineable_properties.products) do
					player.insert({name=item.name})  -- The entity by itself is not included in loot list
				end
			end
			entity.destroy()
			--Tell the player why the building has been removed.
			player.print("This building cannot be build outside platform !")
		end
	end		
	
end)


-- In case a recipe is activated midgame and the research is already done, seek where the recipe is unlocked, check if the research is done and unlock the recipe. This function is ressource heavy, shouldn't be called often. 
-- Can be used to hide the recipe if the recipe have been deactivated in config.

script.on_event("bp-reload-recipe", function(event)
	for _,force in pairs(game.forces) do
		if force.recipes["building-platform"] then -- only if the recipe exist
			if(buildingplatform.config.craftable) then 
				local technologyFound = false
				local recipeToUnlocked = false
				local function inEffect(recipeName,effectsArray)
					for _,effect in pairs(effectsArray) do
						if effect['type'] == "unlock-recipe" and effect['recipe'] == recipeName then
							return true
						end
					end
					return false
				end
				--check for building platform default config
				if(force.technologies["concrete"] and inEffect("building-platform",force.technologies["concrete"].effects)) then
					technologyFound = true
					if force.technologies["concrete"].researched then
						recipeToUnlocked = true
					end
				end
				--if not in default config, search where it has been put
				if not technologyFound then
					for _,tech in pairs(force.technologies) do
						if inEffect("building-platform",tech.effects) then
							technologyFound = true
							if tech.researched then
								recipeToUnlocked = true
								break --stop the loop only if the recipe is to be unlocked, if not, maybe another tech that unlock the platform is researched
							end
						end
					end
				end
				if recipeToUnlocked then
					force.recipes["building-platform"].enabled = true
				end
			else --no craftable 
				force.recipes["building-platform"].enabled = false --in case someone want to deactivate the recipe²²
			end
		end
	end
end)



--=========== pattern code ============--

local append = table.insert

local function limit( a, b, v )
	return math.min( b, math.max( a, v ) )
end

local abs = math.abs
local min = math.min
local max = math.max
local pow = math.pow
local rand = math.random
local floor = math.floor

local function Gen1( x, y )
	ret = false
	ret2 = false
	
	r = math.sqrt( x * x + y * y )
	if math.floor( r ) % 150 < limit( 3, 16, 3 + .15 * r * .3 ) and rand() < .85 then
		ret = true
	end
	rmd = r * .01 * 1.0
	if rmd < 1 and rand() < 1.1 - pow( rmd, 4 ) then
		ret = true
	end
	--if 1 - .1 * x < limit( -10, 10, .3 * abs( y ) ) then
	if abs( x ) < 2 + 8 - .005 * abs( y ) and rand() < pow( 1 - .003 * r, .05 ) then ret = true end
	if abs( y ) < 2 + 8 - .005 * abs( x ) and rand() < pow( 1 - .003 * r, .05 ) then ret = true end
	
	--if abs( x ) % 30 < 4 or abs( y ) % 30 < 4 then
	if abs( x ) < 3 or abs( y ) < 3 then
		ret2 = true
	end
	
	if abs( x - y ) < 3 + .0 * r or abs( y + x ) < 3 + .0 * r then
		ret2 = true
	end
	
	if ( max( abs( x ), abs( y ) ) + 50 ) % 130 < 3 then
		ret2 = true
	end
	
	
	if rmd < .2 then
		ret = false
		ret2 = false
	end
	return ret, ret2
	
end

local function make_chunk(event)
	local c = global.saved_config
	
    --if c == nil or terrain_pattern_get == nil or event.surface.name ~= "nauvis" then
    if event.surface.name ~= "nauvis" then
        -- "nauvis" change from EldVarg, to make it compatible with Factorissimo
        return
    end

    local x1 = event.area.left_top.x
    local y1 = event.area.left_top.y
    local x2 = event.area.right_bottom.x
    local y2 = event.area.right_bottom.y
    local surface = event.surface
    local k = 5

    tiles = {}
	
    for x = x1 - k, x2 + k do
        for y = y1 - k, y2 + k do
            --if (not c.check_for_instant_death) or (x * x + y * y > 3) then
            --if (x * x + y * y > 3) then
			
			local old = surface.get_tile(x, y).name
			--local new = 'building-platform'--terrain_pattern_get(x, y)
			new = nil
			g1, g2 = Gen1( x, y )
			
			if g1 == true then
				new = 'building-platform'
			end
			if g2 == true then
				new = 'building-platform2'
				
			end
			if not (old == "water" or old == "deepwater") then
				if not (new == old) and new then
					append(tiles, {name = new, position = {x, y}})
				end	
			end
			--if new == "land" then
			--    if old == "water" or old == "deepwater" then
			--        append(tiles, {name = "grass", position = {x, y}})
			--    end
			--else
			--    if not (new == old) then
			--        append(tiles, {name = new, position = {x, y}})
			--    end
			--end
            
        end
    end

    surface.set_tiles(tiles)
end


local function on_load(event)
	script.on_event(defines.events.on_chunk_generated, make_chunk)
end

local function on_init(event)
    on_load(nil)
end

script.on_init(on_init)
script.on_load(on_load)