--[[
Copyright (C) 2016  Neemys, ShadowTheAge

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

See <LICENSE.txt> for a copy of this license, or see <http://www.gnu.org/licenses/>
]]


-- Block all selected building added by all mod during data-updates phase.




require("mods-config.load-all") --vanilla blocked building and all mods dependent changes for compatibility



local tiles = data.raw["tile"];
local platformTile = tiles[buildingplatform.tileName];
local platformTile2 = tiles["building-platform2"];

data.raw["player"]["player"].running_speed = .15 * .65

-- By adding non craftable tile and restricted building the same collision_mask, the game make building footprint red outside building platform
-- Can't do the same with placeable tile as when tile are  placed and collide with something they just remove it without compensation. So a script take take of placeable tile check (see controls.lua).


local function addLayerToCollision(item)
	if not item  then return end
	if item.collision_mask then
		table.insert(item.collision_mask, "layer-15");
	else
		item.collision_mask = {"layer-15","water-tile","player-layer"} --Most building does not have collision_mask by default and use the game engine to simulate one, so we need to manually add collision with player and water. Moreover when setting a mask we need to have at least two collision layer in it or they don't use it (for performance reason said a dev, reported as not a bug). 
	end
end

--Add collision to mask for non mineable tile (except platform)
for _,item in pairs(tiles) do
	if item ~= platformTile and not item.minable then	--and item ~= platformTile2 for placement mask
		addLayerToCollision(item);
	end
end


--add to all building marked as blocked the mask to block their placement.

--"block all but" rules
for name,tablePattern in pairs(buildingplatform.buildingsAllowed) do
	local list = data.raw[name];
	if #tablePattern == 0 then --no pattern at all, block all
		for name,item in pairs(list) do
			addLayerToCollision(item);
		end
	else
		log(name);
		log(serpent.block(tablePattern));
		for itemName,item in pairs(list) do
			local matchAtLeastOneTime = false
			for _,pattern in pairs(tablePattern) do
				if( string.match(itemName,pattern) == itemName) then
					matchAtLeastOneTime = true
				end
			end
			if(not matchAtLeastOneTime) then
				addLayerToCollision(item);
			end
		end
	end
end

--"block only" rules
for name,tablePattern in pairs(buildingplatform.buildingsDisallowed) do
	if(not buildingplatform.buildingsAllowed[name]) then --if all, or most building are blocked, "block only" rules are ignored.
		local list = data.raw[name];
		for itemName,item in pairs(list) do
			local matchAtLeastOneTime = false
			for _,pattern in pairs(tablePattern) do
				if( string.match(itemName,pattern) == itemName) then
					matchAtLeastOneTime = true
				end
				if(matchAtLeastOneTime) then
					addLayerToCollision(item);
				end
			end
		end
	end
end