--[[
Copyright (C) 2016  Neemys

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

See <LICENSE.txt> for a copy of this license, or see <http://www.gnu.org/licenses/>
]]

if(buildingplatform.config.craftable) then

	--create building platform item
	local itemConcrete
	if(data.raw["item"]["concrete"]) then --get actual concrete item if possible
		itemConcrete = table.deepcopy(data.raw["item"]["concrete"])
	else -- last check on item concrete data in 0.13.20
		itemConcrete = {
			type = "item",
			flags = {"goes-to-main-inventory"},
			subgroup = "terrain",
			stack_size = 100,
		}
	end
	itemConcrete.name = "building-platform"
	itemConcrete.icon = buildingplatform.modRoot.."/graphics/building-platform/building-platform-icon.png"
	itemConcrete.order = "b[concrete]"
	itemConcrete.place_as_tile = {
		result = buildingplatform.tileName,
		condition_size = 4,
		condition = { "water-tile" }
	}
	
	--create recipe
	local buildingPlatformRecipe
	if(data.raw["recipe"]["concrete"]) then --get actual recipe if possible
		buildingPlatformRecipe = table.deepcopy(data.raw["recipe"]["concrete"])
	else --last check on concrete recipe in 0.13.20
		buildingPlatformRecipe = {
			type = "recipe",
			category = "crafting-with-fluid",
		}
	end
	
	local switchValue = {
		["normal"] =  {
			  {"stone-brick", 70},
			  {"stone", 100},
			  {type="fluid", name="water", amount=100}
		  },
		["normal-expensive"] =  {
			  {"stone-brick", 700},
			  {"stone", 1000},
			  {type="fluid", name="water", amount=1000}
		  },
		["electronic"] =  {
			  {"stone-brick", 35},
			  {"advanced-circuit", 50},
			  {"stone", 50},
			  {type="fluid", name="lubricant", amount=8}
		  },
		["electronic-expensive"] =  {
			  {"stone-brick", 350},
			  {"advanced-circuit", 500},
			  {"stone", 500},
			  {type="fluid", name="lubricant", amount=80}
		  },
		["cheap"] =  {
			  {"stone", 10},
		  },
	}
	
	buildingPlatformRecipe.ingredients = switchValue[buildingplatform.config.recipe]
	if(not buildingPlatformRecipe.ingredients) then buildingPlatformRecipe.ingredients = switchValue["normal"] end
	
	buildingPlatformRecipe.name = "building-platform"
	buildingPlatformRecipe.result = "building-platform"
	buildingPlatformRecipe.result_count = 1
	buildingPlatformRecipe.energy_required = 300
	
	--add tech unlocking for the platform
	if(data.raw['technology']['concrete']) then
		table.insert(data.raw['technology']['concrete'].effects,
		{
			type = "unlock-recipe",
			recipe = "building-platform"
		})
		buildingPlatformRecipe.enabled=false
	else
		buildingPlatformRecipe.enabled=true
	end
	
	--add recipe and item to game
	data:extend({buildingPlatformRecipe,itemConcrete})
end